info "2 - etcd"

check_2_1="2.1  - Ensure that the --cert-file and --key-file arguments are set as appropriate (Not Scored)"
info "$check_2_1"

check_2_2="2.2  - Ensure that the --client-cert-auth argument is set to true (Not Scored)"
info "$check_2_2"

check_2_3="2.3  - Ensure that the --auto-tls argument is not set to true (Not Scored)"
info "$check_2_3"

check_2_4="2.4  - Ensure that the --peer-cert-file and --peer-key-file arguments are set as appropriate (Not Scored)"
info "$check_2_4"

check_2_5="2.5  - Ensure that the --peer-client-cert-auth argument is set to true (Not Scored)"
info "$check_2_5"

check_2_6="2.6  - Ensure that the --peer-auto-tls argument is not set to true (Not Scored)"
info "$check_2_6"

check_2_7="2.7  - Ensure that a unique Certificate Authority is used for etcd (Not Scored)"
info "$check_2_7"
